<?php

namespace Alive\Dm;

class Ajax
{

    const ANON_AUTH_COOKIE = "ALIVE_DM_ANON";
    const CARD_AUTH_KEY = "ALIVE_DM_CARD_AUTH";
    const END_OF_UNIX_EPOCH = 2147483647;

    private $dm;

    public function __construct()
    {
        $this->dm = new Discount_Manager_Client(Config::url(), Config::username(), Config::password(), Config::api_gateway_url(), Config::api_access_key_id(), Config::api_secret_access_key());
    }

    public function init()
    {
        foreach (get_class_methods($this) as $method) {
            if (preg_match('/^(.*)_ajax$/', $method)) {
                $action = substr($method, 0, strlen($method) - 5);

                add_action('wp_ajax_' . $action, array($this, $method));
                add_action('wp_ajax_nopriv_' . $action, array($this, $method));
            }
        }
    }

    /*****************************************************************
     *
     *                      Ajax Handlers
     *
     ****************************************************************/

    /**
     * Handle ajax call for suggestions.
     */
    public function suggestions_ajax()
    {
        $term = !empty($_GET['term']) ? $_GET['term'] : null;
        $sugggestions = $this->dm->get_suggestions($term);
        $this->output_json($sugggestions->suggestions);
    }

    /*
     *  no parameters
     */
    public function categories_ajax()
    {
        $response = $this->dm->get_categories();
        $this->output_json($response->items);
    }

    /*
     * category_id - int
     */
    public function category_ajax()
    {
        $category_id = intval($_POST['category_id']);
        $this->output_json($this->dm->get_category($category_id));
    }

    public function comments_ajax()
    {
        $providerId = intval($_GET['providerId']);
        $offset = intval($_GET['offset']);
        $limit = intval($_GET['limit']);
        $this->output_json($this->add_photos_to_comment_consumers($this->dm->get_comments($providerId, $offset, $limit)));
    }

    /*
     * providerId - int
     */
    public function add_provider_view_ajax()
    {
        $providerId = intval($_POST['providerId']);
        $this->output_json($this->dm->add_provider_view($providerId));
    }

    /*
     * providerId - int
     */
    public function provider_ajax()
    {
        $providerId = intval($_GET['providerId']);
        $this->output_json($this->dm->get_provider($providerId));
    }

    /*
     * name   - string
     * status - string
     */
    public function providers_count_ajax()
    {
        $params = array();
        $params['name'] = isset($_POST['name']) ? $_POST['name'] : null;
        $params['status'] = isset($_POST['status']) ? $_POST['status'] : null;
        $this->output_json($this->dm->get_providers_count($params));
    }

    /*
     * providerId - int
     * value       - int
     */
    public function rate_provider_ajax()
    {
        $providerId = intval($_POST['providerId']);
        $value = intval($_POST['value']);
        $user_agent = $_SERVER['HTTP_USER_AGENT'];
        $ip = $_SERVER['REMOTE_ADDR'];
        $consumer = $this->get_anonymous_consumer();
        $ratings = $this->dm->get_ratings($providerId, $consumer->id);
        if (count($ratings->items) > 0) {
            // Ignore rating if the user has already rated.
            return;
        }
        $this->output_json(
            $this->dm->rate_provider($providerId, $value, $consumer->id, $ip, $user_agent)
        );
    }

    private function get_anonymous_consumer()
    {
        if (!empty($_COOKIE[self::ANON_AUTH_COOKIE])) {
            $consumer = $this->dm->get_consumer("anonymous:" . $_COOKIE[self::ANON_AUTH_COOKIE]);
        }
        if (empty($consumer)) {
            $consumer = $this->create_anonymous_consumer();
            setcookie(self::ANON_AUTH_COOKIE, $consumer->token, self::END_OF_UNIX_EPOCH);
        }
        if (is_user_logged_in() || !empty($_SESSION[self::CARD_AUTH_KEY])) {
            if (is_user_logged_in()) {
                $externalId = "mujisic:" . wp_get_current_user()->ID;
                if (empty($consumer->replacedBy) || $consumer->replacedBy->externalId != $externalId) {
                    $this->dm->replace_consumer($consumer, $externalId);
                }
            } else {
                if (empty($consumer->replacedBy)) {
                    $this->dm->replace_consumer($consumer, $_SESSION[self::CARD_AUTH_KEY]);
                }
            }
        }
        return $consumer;
    }

    /*
     *
     */
    public function comment_ajax()
    {
        $errors = array();
        $username = !empty($_POST['username']) ? $_POST['username'] : null;
        $username = stripslashes($username);
        $cardnumber = !empty($_POST['cardnumber']) ? $_POST['cardnumber'] : null;
        $consumer = $this->get_consumer($username, $cardnumber);
        $text = !empty($_POST['text']) ? stripslashes($_POST['text']) : null;
        $rating = !empty($_POST['rating']) ? $_POST['rating'] : null;
        $providerId = intval($_POST['providerId']);
        $user_agent = $_SERVER['HTTP_USER_AGENT'];
        $ip = $_SERVER['REMOTE_ADDR'];
        $rating_id = null;
        if (!empty($rating)) {
            $rating = $this->dm->rate_provider($providerId, $rating, $consumer->id, $ip, $user_agent);
            if ($rating != -1) {
                $rating_id = $rating->id;
            }
        }
        $res = $this->dm->create_comment($text, $ip, $user_agent, $providerId, $consumer->id, $rating_id);
        return $res->items;
    }

    /*
    *
    */
    public function card_verification_ajax()
    {
        $discountId = !empty($_POST['discountId']) ? intval($_POST['discountId']) : null;
        $providerId = !empty($_POST['providerId']) ? intval($_POST['providerId']) : null;
        $cardNumber = !empty($_POST['cardNumber']) ? $_POST['cardNumber'] : null;
        $cardholderName = !empty($_POST['cardholderName']) ? $_POST['cardholderName'] : null;
        $locale = !empty($_POST['locale']) ? $_POST['locale'] : null;

        $params = [];
        $params['cardholder_name'] = $cardholderName;
        $params['card_number'] = $cardNumber;
        $params['provider_id'] = $providerId;

        $response = $this->dm->get_card_verification($params);

        if ($response === null || ($response->message === 'Invalid data.' || $response->data->result === 'FAILED')) {
            status_header(200);
            $this->output_json([
                'result' => 'FAILED',
                'reason' => 'INVALID_NUMBER',
            ]);
            wp_die();
        } elseif ($response->data->result !== 'SUCCESSFUL') {
            status_header(200);
            $this->output_json([
                'result' => 'FAILED',
                'reason' => 'OTHER',
            ]);
            wp_die();
        }

        $discountMatch = false;
        foreach ($response->data->eligible_discounts as $data) {
            if ($data === $discountId) {
                $discountMatch = true;
                break;
            }
        }

        if (!$discountMatch) {
            status_header(200);
            $this->output_json([
                'result' => 'FAILED',
                'reason' => 'NOT_ELIGIBLE',
            ]);
            wp_die();
        }

        $params = [];
        $params['verification_id'] = $response->data->verification_id;
        $params['discount_id'] = $discountId;

        $response = $this->dm->generate_discount_url($params, $locale);

        if ($response->message !== 'ok') {
            status_header(200);
            $this->output_json([
                'result' => 'FAILED',
                'reason' => 'OTHER',
            ]);
            wp_die();
        }

        status_header(200);
        $this->output_json([
            'result' => 'SUCCESSFUL',
            'url' => $response->data->url,
        ]);
        wp_die();
    }

    /*
    *
    */
    public function category_icons_ajax()
    {
        $categories = $this->dm->get_categories();
        $icons = [];

        foreach ($categories->items as $category) {
            if ($category->icon !== null) {
                $item['icon'] = $category->icon;
            } else {
                $item['icon'] = 'eshop';
            }

            $item['default'] = $category->name;
            foreach ($category->categoryL10ns as $categoryL10ns) {
                $item[$categoryL10ns->languageCode] = $categoryL10ns->name;
            }

            $icons[] = $item;
        }

        status_header(200);
        $this->output_json([
            'result' => 'SUCCESSFUL',
            'icons' => $icons,
        ]);
        wp_die();
    }


    private function get_consumer($username = null, $cardnumber = null)
    {
        if (is_user_logged_in()) {
            $consumer_ext_id = 'mujisic:' . wp_get_current_user()->ID;
        } else {
            $ncdb_client = new Ncdb_Client();
            $card = $ncdb_client->verify_card_number($cardnumber, $username);
            $card_id = $card['cardId'];
            if (empty($card_id)) {
                status_header(403);
                $this->output_json(array('errors' => array(
                    'authentication' => __dm('We could not verify your card. Please check your card number and name.'))));
                wp_die();
            }
            $consumer_ext_id = 'ncdb:' . $card_id;
            $_SESSION[self::CARD_AUTH_KEY] = $consumer_ext_id;
        }
        $consumer = $this->dm->get_consumer($consumer_ext_id);
        if (null === $consumer) {
            if (is_user_logged_in()) {
                $username = wp_get_current_user()->display_name;
            }
            $consumer = $this->dm->create_consumer($username, $consumer_ext_id);
        }
        if (!empty($_COOKIE[self::ANON_AUTH_COOKIE])) {
            $anon_consumer = $this->dm->get_consumer("anonymous:" . $_COOKIE[self::ANON_AUTH_COOKIE]);
            if (empty($anon_consumer->replacedBy) || (is_user_logged_in() && $anon_consumer->replacedBy->id != $consumer->id)) {
                $this->dm->replace_consumer($anon_consumer, $consumer->externalId);
            }
        }
        return $consumer;
    }

    private function create_anonymous_consumer()
    {
        $username = "anonymous";
        $token = $this->generate_token();
        $external_id = "anonymous:$token";
        return $this->dm->create_consumer($username, $external_id, $token);
    }

    private function generate_token($length = 40)
    {
        $alphabet = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
        $result = "";
        for ($i = 0; $i < $length; $i++) {
            $result .= $alphabet[rand(0, strlen($alphabet))];
        }
        return $result;
    }

    /*
     * name      - string
     * status    - ACTIVE, INACTIVE
     * sortKey   - string
     * sortOrder - ASC | DESC
     * offset    - int
     * limit     - int
     */
    public function providers_ajax()
    {
        $params = array();
        $params['text'] = !empty($_GET['text']) ? $_GET['text'] : null;
        $params['cardTypes'] = !empty($_GET['cardTypes']) ? $_GET['cardTypes'] : null;
        $params['categoryIds'] = !empty($_GET['categoryIds']) ? $_GET['categoryIds'] : null;
        $params['sortKey'] = !empty($_GET['sortKey']) ? $_GET['sortKey'] : null;
        $params['sortOrder'] = !empty($_GET['sortOrder']) ? $_GET['sortOrder'] : null;
        $params['location'] = !empty($_GET['location']) ? $_GET['location'] : null;
        $params['seed'] = !empty($_GET['seed']) ? $_GET['seed'] : null;
        $params['offset'] = !empty($_GET['offset']) ? $_GET['offset'] : null;
        $params['limit'] = !empty($_GET['limit']) ? $_GET['limit'] : null;
        $params['facets'] = !empty($_GET['facets']) ? $_GET['facets'] : null;
        $params['locale'] = substr(get_locale(), 0, 2);
        $this->output_json($this->dm->get_providers($params));
    }

    private function add_photo_to_consumer($consumer)
    {
        if (is_object($consumer)) {
            $externalIdParts = explode(':', $consumer->externalId);
            if ($externalIdParts[0] == 'mujisic') {
                $consumer->photoUrl = get_user_meta($externalIdParts[1], 'profilepicture', true);
            }
        }
        return $consumer;
    }

    private function add_photos_to_comment_consumers($comments)
    {
        if (is_object($comments)) {
            foreach ($comments->items as &$comment) {
                $this->add_photo_to_consumer($comment->consumer);
            }
            if (!empty($comments->replies)) {
                foreach ($comment->replies as &$reply) {
                    $this->add_photo_to_consumer($reply->consumer);
                }
            }
        }
        return $comments;
    }

    public function branches_ajax()
    {
        $params = array();
        $params['text'] = !empty($_GET['text']) ? $_GET['text'] : null;
        $params['cardTypes'] = !empty($_GET['cardTypes']) ? $_GET['cardTypes'] : null;
        $params['categoryIds'] = !empty($_GET['categoryIds']) ? $_GET['categoryIds'] : null;
        $params['boundingBox'] = !empty($_GET['boundingBox']) ? $_GET['boundingBox'] : null;
        $params['availableOnline'] = !empty($_GET['availableOnline']) ? $_GET['availableOnline'] : null;
        $params['facets'] = !empty($_GET['facets']) ? $_GET['facets'] : null;
        $params['sortKey'] = !empty($_GET['sortKey']) ? $_GET['sortKey'] : null;
        $params['sortOrder'] = !empty($_GET['sortOrder']) ? $_GET['sortOrder'] : null;
        $params['location'] = !empty($_GET['location']) ? $_GET['location'] : null;
        $params['offset'] = !empty($_GET['offset']) ? $_GET['offset'] : null;
        $params['limit'] = !empty($_GET['limit']) ? $_GET['limit'] : null;
        $this->output_json($this->dm->get_branches($params));
    }

    /*****************************************************************
     *
     *                      Helper/Private methods
     *
     ****************************************************************/

    private function output_json($data)
    {
        if (is_int($data) and -1 == $data) {
            throw new \Exception('AJAX failed.');
        }
        echo json_encode($data);
        die();
    }
}
