<?php

namespace Alive\Dm;

class Validator_Sanitizer {

    public function __construct($name, $getter, $validate = array(), $sanitize = array()) {
        $this->name = $name;
        $this->getter = $getter;

        foreach ($validate as $what => $if_not) {
            $this->ensure_that($what, $if_not);
        }

        foreach ($sanitize as $v) {
            $this->sanitize($v);
        }
    }

    public function __invoke($value) {
        foreach ($this->ensures as $ensure_that => $if_not) {
            if (!call_user_func(array($this, $ensure_that), $value)) {
                $value = call_user_func($this->getter);
                add_settings_error(
                    $this->name,
                    'error',
                    $if_not
                );
                return $value;
            }
        }

        foreach ($this->sanitizes as $filter_by) {
            $value = call_user_func(array($this, $filter_by), $value);
        }

        return $value;
    }

    private $ensures = array();

    public function ensure_that($what, $if_not) {
        if (method_exists($this, $what)) {
            $this->ensures[$what] = $if_not;
        } else {
            throw new \InvalidArgumentException("Cannot ensure that $what, don't know it.");
        }
    }

    private $sanitizes = array();

    public function filter_by($what) {
        if (method_exists($this, $what)) {
            $this->sanitizes[$what] = $if_not;
        } else {
            throw new \InvalidArgumentException("Cannot filter by $what, don't know it.");
        }
    }

    /***************************************************
    *
    *               Validators
    *
    ***************************************************/

    private function is_url($value) {
        return filter_var($value, FILTER_VALIDATE_URL);
    }

    private function is_currency_pos($value) {
        return $value == 1 || $value == 2;
    }

    private function is_locale($value) {
        return preg_match('/^[[:alpha:]]{2}(_[[:alpha:]]{2})?(\.[-_[:alnum:]]+)?(@[[:alpha:]]+)?$/', $value);
    }

}
