<?php

// Define OWN absolute path in order to make wordpress load fake wp-setiings file.
define('ABSPATH', dirname(__FILE__) . '/');

// Include wordpress configuration for database connection settings.
include(dirname(__FILE__) . "/../../../../wp-config.php");
// Use absolute path when using symlink.
//include("{absolute-path-to}/wp-config.php");

/**
 * Proxy for handling ajax requests for the the DM plugin.
 */
class RequestProxy {

    /**
     * @var string Discount manager URL.
     */
    private $dmUrl;

    /**
     * @var string DM technical account username.
     */
    private $dmUsername;

    /**
     * @var string DM technical account password.
     */
    private $dmPassword;

    /**
     * Create new instance of request proxy .
     *
     * @param string $dmUrl Discount manager URL. Never null.
     * @param string $dmUsername DM technical account username. Never null.
     * @param string $dmPassword DM technical account password. Never null.
     */
    public function __construct($dmUrl, $dmUsername, $dmPassword) {
        $this->dmUrl = $dmUrl;
        $this->dmUsername = $dmUsername;
        $this->dmPassword = $dmPassword;
    }


    /**
     * Handle the proxy request.
     * @param string $action Ajax action name as specified for the standard wordpress handler.
     * @param string $params Parameters of the call.
     * @return string Json encoded result.
     */
    public function handle($action, $params, $post) {
        switch ($action) {
            case 'suggestions':
                return $this->suggestions($params['term']);
            case 'branches':
                return $this->getBranches($params);
            case 'provider':
                return $this->getProvider($params['providerId']);
            case 'providers':
                return $this->getProviders($params);
            case 'comments':
                return $this->getComments($params['providerId'],
                        $params['offset'], $params['limit']);
            case 'add_provider_view':
                return $this->addProviderView($post['providerId']);
            default:
                echo "Unsupported action.";
        }
    }

    /**
     * Fetch suggestions from DM and return.
     * @param $term Term to be suggested.
     * @return json List of suggestions from DM.
     */
    private function suggestions($text) {
        $url = $this->dmUrl . (substr($this->dmUrl, -1) == '/' ? "" : "/") . 'api/suggestions?' . http_build_query(array('text' => $text));
        $result = $this->get($url, $this->dmUsername, $this->dmPassword);
        // Transform suggestions - unfortunatelly necessary, becayse dm plugin makes this transformation.
        $suggestions = json_decode($result);
        $suggestions = $suggestions->suggestions;
        return json_encode($suggestions);
    }

    /**
     * Get branches based on given parameters
     * @param $params url prameters
     * @return json branches.
     */
    function getBranches($params) {

        $valid_keys = array('text', 'cardTypes', 'categoryIds', 'boundingBox', 'availableOnline',
            'facets', 'sortKey', 'sortOrder', 'location', 'offset', 'limit');
        $queryParams = array('visible' => 'true');
        foreach ($valid_keys as $key) {
            if (isset($params[$key]) && !empty($params[$key])) {
                $queryParams[$key] = $params[$key];
            }
        }
        $url = $this->dmUrl . (substr($this->dmUrl, -1) == '/' ? "" : "/") . 'api/branches?' . http_build_query($queryParams);
        return $this->get($url, $this->dmUsername, $this->dmPassword);
    }

    /**
     * Fetch single provider form DM.
     * @param int $providerId providers id
     * @return json DM responce
     */
    private function getProvider($providerId) {
        $url = $this->dmUrl . (substr($this->dmUrl, -1) == '/' ? "" : "/") . 'api/providers/' . htmlentities($providerId);
        return $this->get($url, $this->dmUsername, $this->dmPassword);
    }

    /**
     * Fetch providers from DM based on query.
     * @param array $params GET query param
     * @return json DM responce
     */
    private function getProviders($params) {
        $valid_keys = array('text', 'categoryIds', 'cardTypes', 'sortKey', 'sortOrder',
            'location', 'offset', 'limit', 'facets', 'seed', 'locale');
        $queryParams = array('visible' => 'true');
        foreach ($valid_keys as $key) {
            if (isset($params[$key]) && !empty($params[$key])) {
                $queryParams[$key] = $params[$key];
            }
        }
        $url = $this->dmUrl . (substr($this->dmUrl, -1) == '/' ? "" : "/") . 'api/providers?' . http_build_query($queryParams);
        return $this->get($url, $this->dmUsername, $this->dmPassword);
    }

    /**
     * Get comments of given provider from DM.
     * @param int $providerId Provider identifier.
     * @param int $offset Offset.
     * @param int $limit limit.
     * @return json DM responce.
     */
    function getComments($providerId, $offset, $limit) {
        $queryParams = array('offset' => $offset, 'limit' => $limit);
        $url = $this->dmUrl . (substr($this->dmUrl, -1) == '/' ? "" : "/") . 'api/providers/' . htmlentities($providerId) . '/comments?' . http_build_query($queryParams);
        return $this->get($url, $this->dmUsername, $this->dmPassword);
    }

    /**
     * Increase provider view count.
     * @param int $providerId Provider identifier.
     * @return json DM responce.
     */
    function addProviderView($providerId) {

        $providerId = 15;
        $providerView = array(
            'userAgent' => $_SERVER['HTTP_USER_AGENT'],
            'ip' => $_SERVER['REMOTE_ADDR'],
        );
        $url = $this->dmUrl . (substr($this->dmUrl, -1) == '/' ? "" : "/") . 'api/providers/' . htmlentities($providerId) . '/views';
        return $this->post($url, $this->dmUsername, $this->dmPassword,
                json_encode($providerView));
    }

    /**
     * Make GET request o given URL with the authentication headers.
     */
    private function get($url, $username, $password) {
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_HEADER, FALSE);
        curl_setopt($ch, CURLOPT_USERPWD, $username . ":" . $password);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
        $result = curl_exec($ch);
        curl_close($ch);
        return $result;
    }

     /**
     * Make POST request o given URL with the authentication headers.
     */
    private function post($url, $username, $password, $body) {
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_POST);
        curl_setopt($ch, CURLOPT_USERPWD, $username . ":" . $password);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($ch, CURLOPT_POSTFIELDS, $body);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER,
            array(
            'Content-Type: application/json',
            'Content-Length: ' . strlen($body))
        );

        $result = curl_exec($ch);
        curl_close($ch);
        return $result;
    }

}

/**
 * Helper class for reading wordpress options.
 */
class WpOptions {

    private static $cachePrefix = 'alive-dm-plugin-';

    /**
     * Fetch wordpress oprtion and try to cache them.
     */
    public function get($name, $domain = null) {
        $key = $name;
        if (!empty($domain)) {
            $key = $domain . '-' . $key;
        }
        $value = $this->getFromCache($key);
        if ($value === false) {
            $value = $this->getFromDb($key);
            $this->writeToCache($key, $value);
        }
        return $value;
    }

    /**
     * Get option from WP database.
     * @param string $key Option key.
     * @return mixed Option value or null if option does not exist.
     */
    private function getFromDb($key) {
        // Include table prefix.
        global $table_prefix;
        // Create database connection
        $link = mysqli_connect(DB_HOST, DB_USER, DB_PASSWORD);
        // Select database.
        mysqli_select_db($link, DB_NAME);
        // Get DB result.
        $query = "SELECT option_value FROM " . $table_prefix . "options WHERE option_name = '" . mysqli_real_escape_string($link, $key) . "' LIMIT 1";
        $result = mysqli_query($link, $query);
        $option = mysqli_fetch_array($result, MYSQLI_ASSOC);
        // Free resources.
        mysqli_free_result($result);
        mysqli_close($link);
        // Get option value.
        if (!empty($option) && !empty($option['option_value'])) {
            return $option['option_value'];
        }
        return null;
    }

    /**
     * Try to get valuye from the cache.
     * @param $key Key of the option. Never null.
     * @return mixed|boolean Option value or false in case of cache miss.
     */
    private function getFromCache($key) {
        // If there is APC installed cache the value.
        if (function_exists('apc_fetch')) {
            return apc_fetch(self::$cachePrefix . $key);
        }
        return false;
    }

    /**
     * Write value to the cache. (Cached for one day).
     * @param string $key Key of the option. Never null.
     * @param unknown $value Value of the option. Can be null.
     */
    private function writeToCache($key, $value) {
        if (function_exists('apc_store')) {
            apc_store(self::$cachePrefix . $key, $value, 86400);
        }
    }
}

// Read necessary options from the configuration file.
$options = new WpOptions();
$dmUrl = $options->get('dm-url', 'alive-dm-plugin');
$dmUsername = $options->get('dm-username', 'alive-dm-plugin');
$dmPassword = $options->get('dm-password', 'alive-dm-plugin');

// Get request parameters.
$action = $_GET['action'];
$params = $_GET;
unset($params['action']);
$post = $_POST;

// Return suggestions.
header("Content-Type: application/json");
$requestProxy = new RequestProxy($dmUrl, $dmUsername, $dmPassword);
echo $requestProxy->handle($action, $params, $post);
exit;
