(function ($) {
    "use strict";
    /**
     * Class responsible for handling of the discount finder.
     */
    window.DmDiscountShortcut = function (wrapper, config) {
        this.constructor.call(this, wrapper, config);
    }
    $.extend(DmDiscountShortcut.prototype, {
       
        /**
         * Wrapper element of the discount finder DOM.
         */
        wrapper: null,
        /**
         * Base URL. Should be replaced by client.
         */
        ajaxUrl: null,
        
        /**
         * Search criteria handler.
         */
        searching: null,
        /**
         * Discount finder constructor.
         * @param wrapper Element wrapping discount finder HTML structure.
         * @param config Configuration attributes.
         */
        constructor: function (wrapper, config) {
            if (!config.ajaxUrl) {
                throw new Error("Configuration has to contain ajaxUrl for wordpress ajax calls.");
            }
            // Set configuration variables.
            this.ajaxUrl = config.ajaxUrl;
            this.wrapper = wrapper;
            // Initializae handlers.
            this.searching = new DmShortcutSearching(wrapper, this.ajaxUrl);
        },
    });

    /**
     * Class responsible for handling search criteria and their changes.
     */
    window.DmShortcutSearching = function (wrapper, ajaxUrl) {
        this.constructor.call(this, wrapper, ajaxUrl);
    }
    $.extend(DmShortcutSearching.prototype, {
        /**
         * URL for the AJAX calls.
         */
        ajaxUrl: null,
        /**
         * Available card types.
         */
        //~ Values currently set to the filter.
        text: null,        
        //~ Search filter fields.
        textInput: null,
        //~ Helper variables.
        searchTimeout: null,
        /**
         * Construct the pagination object.
         * @param wrapper Wrapper of the finder.
         * @param query Array representation of the query parameters.
         */
        constructor: function (wrapper, ajaxUrl) {
            this.ajaxUrl = ajaxUrl;
            this.init(wrapper);
        },
        /**
         * Initialize context.
         * @pararm wrapper Wrapper of the finder plugin.
         */
        init: function (wrapper) {
            var that = this;
            
            // Initialize text input handling.
            this.textInput = $('[data-dm-target="filter-text"]', this.wrapper);
            this.textInput.on('change', function (event) {
                if (that.text != $(event.target).val()) {
                    that.text = $(event.target).val();
                    that.onChange();
                }
                return false;
            });
          
            // Initialize suggestions handler.
            this.initSuggestions();
            // Store available card types.
            $('[data-dm-target="filter-card-types-card-type"]').each(function (index, element) {
                that.cardTypes.push($(element).data('dm-card-type'));
            });
        },
        /**
         * Handle change of the text search.
         */
        handleTextChange: function (event) {
            var that = this;
            var code = event.keyCode || event.which;
            var text = this.textInput.val().trim();
            // Stop prevous search timeout.
            clearTimeout(this.searchTimeout);
            // Do not perform search if the text didn't change.
            if (this.text == text) {
                return false;
            }
            // On enter search immediatelly
            if (code == 13) {
                this.text = text;
                this.onChange();
                return false;
            }
            // Do not perform search if the string is too short.
            if (text.length < 3) {
                return false;
            }
            // Postpone the live search.
            this.searchTimeout = setTimeout(function () {
                if (text == that.textInput.val().trim()) {
                    that.text = that.textInput.val().trim();
                    that.onChange();
                }
            }, 2000 /* 2s */);
        },
        /**
         * Set searched text.
         */
        setText: function (text) {
            this.text = text;
            this.textInput.val(text);
        },
        /**
         * Initialize autocomplete functionality.
         */
        initSuggestions: function () {
            var that = this;
            $('[data-dm-target="filter-text"]').autocomplete({
                minLength: 1,
                delay: 200,
                source: this.ajaxUrl + "?action=suggestions",
                select: function (event, data) {
                    // Search immediately when item is selected.
                    that.setText(data.item.text);
                    $('#dm-discounts-shortcut form').submit();
                    return false;
                },
                focus: function (event, ui) {
                    // Do not perform searches when user is selecting a value.
                },
                open: function (event, ui) {
                    // Fix of width. Needed for good responsive behavior.
                    $(".dm-suggestions").css("width", $(event.target).innerWidth() + "px");
                    return false;
                },
                messages: {
                    noResults: '',
                    results: function () {
                    }
                },
                appendTo: "#dm-discounts-shortcut .dm-text-search-options"
            }).data("ui-autocomplete")._renderItem = function (ul, item) {
                ul.addClass("dm-suggestions");
                return $("<li class=\"dm-suggestion\">")
                        .data("item.autocomplete", item)
                        .append($("<a></a>").text(item.text))
                        .appendTo(ul);
            };

            $('[data-dm-target="filter-text"]').keyup(function (event) {
                var code = event.keyCode || event.which;
                if (code == 13) {
                    // Close autocomplete on enter
                    $(this).autocomplete("close");
                }
            });
        },
       
    });

})(jQuery);
